"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.share = void 0;
const Observable_js_1 = require("../Observable.js");
const Subject_js_1 = require("../Subject.js");
function share(options = {}) {
    const { connector = () => new Subject_js_1.Subject(), resetOnError = true, resetOnComplete = true, resetOnRefCountZero = true } = options;
    return (source) => {
        let connection;
        let resetConnection;
        let subject;
        let refCount = 0;
        let hasCompleted = false;
        let hasErrored = false;
        const cancelReset = () => {
            resetConnection?.unsubscribe();
            resetConnection = undefined;
        };
        const reset = () => {
            cancelReset();
            connection = subject = undefined;
            hasCompleted = hasErrored = false;
        };
        const resetAndUnsubscribe = () => {
            const conn = connection;
            reset();
            conn?.unsubscribe();
        };
        return new Observable_js_1.Observable((subscriber) => {
            refCount++;
            if (!hasErrored && !hasCompleted) {
                cancelReset();
            }
            const dest = (subject = subject ?? connector());
            subscriber.add(() => {
                refCount--;
                if (refCount === 0 && !hasErrored && !hasCompleted) {
                    resetConnection = handleReset(resetAndUnsubscribe, resetOnRefCountZero);
                }
            });
            dest.subscribe(subscriber);
            if (!connection &&
                refCount > 0) {
                connection = new Observable_js_1.Subscriber({
                    next: (value) => dest.next(value),
                    error: (err) => {
                        hasErrored = true;
                        cancelReset();
                        resetConnection = handleReset(reset, resetOnError, err);
                        dest.error(err);
                    },
                    complete: () => {
                        hasCompleted = true;
                        cancelReset();
                        resetConnection = handleReset(reset, resetOnComplete);
                        dest.complete();
                    },
                });
                (0, Observable_js_1.from)(source).subscribe(connection);
            }
        });
    };
}
exports.share = share;
function handleReset(reset, on, ...args) {
    if (on === true) {
        reset();
        return;
    }
    if (on === false) {
        return;
    }
    const onSubscriber = new Observable_js_1.Subscriber({
        next: () => {
            onSubscriber.unsubscribe();
            reset();
        },
    });
    return (0, Observable_js_1.from)(on(...args)).subscribe(onSubscriber);
}
//# sourceMappingURL=share.js.map